@@ -23,6 +23,7 @@ Some slight modifications, but at some point
 we should look at maybe pushing this up to Oslo
 """
 
+from __future__ import annotations  # Remove when only supporting python 3.9+
 
 import contextlib
 import errno
@@ -31,8 +32,7 @@ import math
 import os
 import re
 import tempfile
-from typing import (ContextManager, Dict, Generator,  # noqa: H301
-                    List, Optional, Tuple)
+from typing import ContextManager, Generator, Optional  # noqa: H301
 
 import cryptography
 from cursive import exception as cursive_exception
@@ -73,33 +73,13 @@ image_opts = [
     cfg.IntOpt('image_conversion_address_space_limit',
                default=1,
                help='Address space limit in gigabytes to convert the image'),
-    cfg.ListOpt('vmdk_allowed_types',
-                default=['streamOptimized', 'monolithicSparse'],
-                help='A list of strings describing the VMDK createType '
-                'subformats that are allowed.  We recommend that you only '
-                'include single-file-with-sparse-header variants to avoid '
-                'potential host file exposure when processing named extents '
-                'when an image is converted to raw format as it is written '
-                'to a volume.  If this list is empty, no VMDK images are '
-                'allowed.'),
-    cfg.ListOpt('reserved_image_namespaces',
-                help='List of reserved image namespaces that should be '
-                     'filtered out when uploading a volume as an image back '
-                     'to Glance. When a volume is created from an image, '
-                     'Cinder stores the image properties as volume '
-                     'image metadata, and if the volume is later uploaded as '
-                     'an image, Cinder will add these properties when it '
-                     'creates the image in Glance. This can cause problems '
-                     'for image metadata that are in namespaces that glance '
-                     'reserves for itself, or when properties (such as an '
-                     'image signature) cannot apply to the new image, or when '
-                     'an operator has configured glance property protections '
-                     'to make some image properties read-only. Cinder will '
-                     '*always* filter out image metadata in the namespaces '
-                     '`os_glance` and `img_signature`; this configuration '
-                     'option allows operators to specify *additional* '
-                     'namespaces to be excluded.',
-                default=[]),
+    cfg.BoolOpt('image_conversion_disable',
+                default=False,
+                help='Disallow image conversion when creating a volume from '
+                'an image and when uploading a volume as an image. Image '
+                'conversion consumes a large amount of system resources and '
+                'can cause performance problems on the cinder-volume node. '
+                'When set True, this option disables image conversion.'),
 ]
 
 CONF = cfg.CONF
@@ -124,8 +104,6 @@ QEMU_IMG_MIN_CONVERT_LUKS_VERSION = '2.10'
 
 COMPRESSIBLE_IMAGE_FORMATS = ('qcow2',)
 
-GLANCE_RESERVED_NAMESPACES = ["os_glance", "img_signature"]
-
 
 def validate_stores_id(context: context.RequestContext,
                        image_service_store_id: str) -> None:
@@ -157,7 +135,7 @@ def qemu_img_info(path: str,
                   run_as_root: bool = True,
                   force_share: bool = False) -> imageutils.QemuImgInfo:
     """Return an object containing the parsed output from qemu-img info."""
-    cmd = ['env', 'LC_ALL=C', 'qemu-img', 'info', '--output=json']
+    cmd = ['env', 'LC_ALL=C', 'qemu-img', 'info']
     if force_share:
         if qemu_img_supports_force_share():
             cmd.append('--force-share')
@@ -171,7 +149,7 @@ def qemu_img_info(path: str,
         cmd = cmd[2:]
     out, _err = utils.execute(*cmd, run_as_root=run_as_root,
                               prlimit=QEMU_IMG_LIMITS)
-    info = imageutils.QemuImgInfo(out, format='json')
+    info = imageutils.QemuImgInfo(out)
 
     # From Cinder's point of view, any 'luks' formatted images
     # should be treated as 'raw'.
@@ -181,7 +159,7 @@ def qemu_img_info(path: str,
     return info
 
 
-def get_qemu_img_version() -> Optional[List[int]]:
+def get_qemu_img_version() -> Optional[list[int]]:
     """The qemu-img version will be cached until the process is restarted."""
 
     global QEMU_IMG_VERSION
@@ -212,11 +190,11 @@ def _get_qemu_convert_luks_cmd(src: str,
                                src_format: Optional[str] = None,
                                out_subformat: Optional[str] = None,
                                cache_mode: Optional[str] = None,
-                               prefix: tuple = None,
+                               prefix: Optional[tuple] = None,
                                cipher_spec: Optional[dict] = None,
                                passphrase_file: Optional[str] = None,
-                               src_passphrase_file: str = None) -> List[str]:
-
+                               src_passphrase_file: Optional[str] = None) \
+        -> list[str]:
     cmd = ['qemu-img', 'convert']
 
     if prefix:
@@ -243,15 +221,15 @@ def _get_qemu_convert_luks_cmd(src: str,
 def _get_qemu_convert_cmd(src: str,
                           dest: str,
                           out_format: str,
-                          src_format: str = None,
-                          out_subformat: str = None,
-                          cache_mode: str = None,
-                          prefix: tuple = None,
-                          cipher_spec: dict = None,
-                          passphrase_file: str = None,
+                          src_format: Optional[str] = None,
+                          out_subformat: Optional[str] = None,
+                          cache_mode: Optional[str] = None,
+                          prefix: Optional[tuple] = None,
+                          cipher_spec: Optional[dict] = None,
+                          passphrase_file: Optional[str] = None,
                           compress: bool = False,
-                          src_passphrase_file: str = None) -> List[str]:
-
+                          src_passphrase_file: Optional[str] = None) \
+        -> list[str]:
     if src_passphrase_file is not None:
         if passphrase_file is None:
             message = _("Can't create unencrypted volume %(format)s "
@@ -317,7 +295,7 @@ def _get_qemu_convert_cmd(src: str,
     return cmd
 
 
-def _get_version_from_string(version_string: str) -> List[int]:
+def _get_version_from_string(version_string: str) -> list[int]:
     return [int(x) for x in version_string.split('.')]
 
 
@@ -343,13 +321,13 @@ def _convert_image(prefix: tuple,
                    source: str,
                    dest: str,
                    out_format: str,
-                   out_subformat: str = None,
-                   src_format: str = None,
+                   out_subformat: Optional[str] = None,
+                   src_format: Optional[str] = None,
                    run_as_root: bool = True,
-                   cipher_spec: dict = None,
-                   passphrase_file: str = None,
+                   cipher_spec: Optional[dict] = None,
+                   passphrase_file: Optional[str] = None,
                    compress: bool = False,
-                   src_passphrase_file: str = None) -> None:
+                   src_passphrase_file: Optional[str] = None) -> None:
     """Convert image to other format.
 
     NOTE: If the qemu-img convert command fails and this function raises an
@@ -450,42 +428,14 @@ def _convert_image(prefix: tuple,
 def convert_image(source: str,
                   dest: str,
                   out_format: str,
-                  out_subformat: str = None,
-                  src_format: str = None,
+                  out_subformat: Optional[str] = None,
+                  src_format: Optional[str] = None,
                   run_as_root: bool = True,
                   throttle=None,
-                  cipher_spec: dict = None,
-                  passphrase_file: str = None,
+                  cipher_spec: Optional[dict] = None,
+                  passphrase_file: Optional[str] = None,
                   compress: bool = False,
-                  src_passphrase_file: str = None,
-                  image_id: str = None,
-                  data: imageutils.QemuImgInfo = None) -> None:
-    """Convert image to other format.
-
-    NOTE: If the qemu-img convert command fails and this function raises an
-    exception, a non-empty dest file may be left in the filesystem.
-    It is the responsibility of the caller to decide what to do with this file.
-
-    :param source: source filename
-    :param dest: destination filename
-    :param out_format: output image format of qemu-img
-    :param out_subformat: output image subformat
-    :param src_format: source image format (use image_utils.fixup_disk_format()
-           to translate from a Glance format to one recognizable by qemu_img)
-    :param run_as_root: run qemu-img as root
-    :param throttle: a cinder.throttling.Throttle object, or None
-    :param cipher_spec: encryption details
-    :param passphrase_file: filename containing luks passphrase
-    :param compress: compress w/ qemu-img when possible (best effort)
-    :param src_passphrase_file: filename containing source volume's
-                                luks passphrase
-    :param image_id: the image ID if this is a Glance image, or None
-    :param data: a imageutils.QemuImgInfo object from this image, or None
-    :raises ImageUnacceptable: when the image fails some format checks
-    :raises ProcessExecutionError: when something goes wrong during conversion
-    """
-    check_image_format(source, src_format, image_id, data, run_as_root)
-
+                  src_passphrase_file: Optional[str] = None) -> None:
     if not throttle:
         throttle = throttling.Throttle.get_default()
     with throttle.subcommand(source, dest) as throttle_cmd:
@@ -504,9 +454,9 @@ def convert_image(source: str,
 def resize_image(source: str,
                  size: int,
                  run_as_root: bool = False,
-                 file_format: str = None) -> None:
+                 file_format: Optional[str] = None) -> None:
     """Changes the virtual size of the image."""
-    cmd: Tuple[str, ...]
+    cmd: tuple[str, ...]
     if file_format:
         cmd = ('qemu-img', 'resize', '-f', file_format, source, '%sG' % size)
     else:
@@ -680,98 +630,6 @@ def get_qemu_data(image_id: str,
     return data
 
 
-def check_vmdk_image(image_id: str, data: imageutils.QemuImgInfo) -> None:
-    """Check some rules about VMDK images.
-
-    Make sure the VMDK subformat (the "createType" in vmware docs)
-    is one that we allow as determined by the 'vmdk_allowed_types'
-    configuration option.  The default set includes only types that
-    do not reference files outside the VMDK file, which can otherwise
-    be used in exploits to expose host information.
-
-    :param image_id: the image id
-    :param data: an imageutils.QemuImgInfo object
-    :raises ImageUnacceptable: when the VMDK createType is not in the
-                               allowed list
-    """
-    allowed_types = CONF.vmdk_allowed_types
-
-    if not len(allowed_types):
-        msg = _('Image is a VMDK, but no VMDK createType is allowed')
-        raise exception.ImageUnacceptable(image_id=image_id, reason=msg)
-
-    try:
-        create_type = data.format_specific['data']['create-type']
-    except KeyError:
-        msg = _('Unable to determine VMDK createType')
-        raise exception.ImageUnacceptable(image_id=image_id, reason=msg)
-    except TypeError:
-        msg = _('Unable to determine VMDK createType as no format-specific '
-                'information is available')
-        raise exception.ImageUnacceptable(image_id=image_id, reason=msg)
-
-    if create_type not in allowed_types:
-        LOG.warning('Refusing to process VMDK file with createType of %r '
-                    'which is not in allowed set of: %s', create_type,
-                    ','.join(allowed_types))
-        msg = _('Invalid VMDK create-type specified')
-        raise exception.ImageUnacceptable(image_id=image_id, reason=msg)
-
-
-def check_image_format(source: str,
-                       src_format: str = None,
-                       image_id: str = None,
-                       data: imageutils.QemuImgInfo = None,
-                       run_as_root: bool = True) -> None:
-    """Do some image format checks.
-
-    Verifies that the src_format matches what qemu-img thinks the image
-    format is, and does some vmdk subformat checks.  See Bug #1996188.
-
-    - Does not check for a qcow2 backing file.
-    - Will make a call out to qemu_img if data is None.
-
-    :param source: filename of the image to check
-    :param src_format: source image format recognized by qemu_img, or None
-    :param image_id: the image ID if this is a Glance image, or None
-    :param data: a imageutils.QemuImgInfo object from this image, or None
-    :param run_as_root: when 'data' is None, call 'qemu-img info' as root
-    :raises ImageUnacceptable: when the image fails some format checks
-    :raises ProcessExecutionError: if 'qemu-img info' fails
-    """
-    if image_id is None:
-        image_id = 'internal image'
-    if data is None:
-        data = qemu_img_info(source, run_as_root=run_as_root)
-
-    if data.file_format is None:
-        raise exception.ImageUnacceptable(
-            reason=_("'qemu-img info' parsing failed."),
-            image_id=image_id)
-
-    if src_format is not None:
-        if src_format.lower() == 'ami':
-            # qemu-img doesn't recognize AMI format; see change Icde4c0f936ce.
-            # We also use lower() here (though nowhere else) to be consistent
-            # with that change.
-            pass
-        elif data.file_format != src_format:
-            LOG.debug("Rejecting image %(image_id)s due to format mismatch. "
-                      "src_format: '%(src)s', but qemu-img info reports: "
-                      "'%(qemu)s'",
-                      {'image_id': image_id,
-                       'src': src_format,
-                       'qemu': data.file_format})
-            msg = _("The image format was claimed to be '%(src)s' but the "
-                    "image data appears to be in a different format.")
-            raise exception.ImageUnacceptable(
-                image_id=image_id,
-                reason=(msg % {'src': src_format}))
-
-    if data.file_format == 'vmdk':
-        check_vmdk_image(image_id, data)
-
-
 def fetch_verify_image(context: context.RequestContext,
                        image_service: glance.GlanceImageService,
                        image_id: str,
@@ -789,10 +647,7 @@ def fetch_verify_image(context: context.RequestContext,
         data = get_qemu_data(image_id, has_meta, format_raw,
                              dest, True)
         # We can only really do verification of the image if we have
-        # qemu data to use.
-        # NOTE: We won't have data if qemu_img is not installed *and* the
-        # disk_format recorded in Glance is raw (otherwise an ImageUnacceptable
-        # would have been raised already).  So this isn't as bad as it looks.
+        # qemu data to use
         if data is not None:
             fmt = data.file_format
             if fmt is None:
@@ -807,11 +662,6 @@ def fetch_verify_image(context: context.RequestContext,
                     reason=(_("fmt=%(fmt)s backed by: %(backing_file)s") %
                             {'fmt': fmt, 'backing_file': backing_file}))
 
-            # a VMDK can have a backing file, but we have to check for
-            # it differently
-            if fmt == 'vmdk':
-                check_vmdk_image(image_id, data)
-
 
 def fetch_to_vhd(context: context.RequestContext,
                  image_service: glance.GlanceImageService,
@@ -842,6 +692,28 @@ def fetch_to_raw(context: context.RequestContext,
                            size=size, run_as_root=run_as_root)
 
 
+def check_image_conversion_disable(disk_format, volume_format, image_id,
+                                   upload=False):
+    if CONF.image_conversion_disable and disk_format != volume_format:
+        if upload:
+            msg = _("Image conversion is disabled. The image disk_format "
+                    "you have requested is '%(disk_format)s', but your "
+                    "volume can only be uploaded in the format "
+                    "'%(volume_format)s'.")
+        else:
+            msg = _("Image conversion is disabled. The volume type you have "
+                    "requested requires that the image it is being created "
+                    "from be in '%(volume_format)s' format, but the image "
+                    "you are using has the disk_format property "
+                    "'%(disk_format)s'. You must use an image with the "
+                    "disk_format property '%(volume_format)s' to create a "
+                    "volume of this type.")
+        raise exception.ImageConversionNotAllowed(
+            reason=msg % {'disk_format': disk_format,
+                          'volume_format': volume_format},
+            image_id=image_id)
+
+
 def fetch_to_volume_format(context: context.RequestContext,
                            image_service: glance.GlanceImageService,
                            image_id: str,
@@ -856,6 +728,9 @@ def fetch_to_volume_format(context: context.RequestContext,
     qemu_img = True
     image_meta = image_service.show(context, image_id)
 
+    check_image_conversion_disable(
+        image_meta['disk_format'], volume_format, image_id, upload=False)
+
     allow_image_compression = CONF.allow_compression_on_image_upload
     if image_meta and (image_meta.get('container_format') == 'compressed'):
         if allow_image_compression is False:
@@ -877,10 +752,6 @@ def fetch_to_volume_format(context: context.RequestContext,
             format_raw = True if image_meta['disk_format'] == 'raw' else False
         except TypeError:
             format_raw = False
-
-        # Probe using the empty tmp file to see if qemu-img is available.
-        # If it's not, and the disk_format recorded in Glance is not 'raw',
-        # this will raise ImageUnacceptable
         data = get_qemu_data(image_id, has_meta, format_raw,
                              tmp, run_as_root)
         if data is None:
@@ -893,21 +764,6 @@ def fetch_to_volume_format(context: context.RequestContext,
         else:
             fetch(context, image_service, image_id, tmp, user_id, project_id)
 
-        # NOTE(ZhengMa): This is used to do image decompression on image
-        # downloading with 'compressed' container_format. It is a
-        # transparent level between original image downloaded from
-        # Glance and Cinder image service. So the source file path is
-        # the same with destination file path.
-        if image_meta.get('container_format') == 'compressed':
-            LOG.debug("Found image with compressed container format")
-            if not accelerator.is_gzip_compressed(tmp):
-                raise exception.ImageUnacceptable(
-                    image_id=image_id,
-                    reason=_("Unsupported compressed image format found. "
-                             "Only gzip is supported currently"))
-            accel = accelerator.ImageAccel(tmp, tmp)
-            accel.decompress_img(run_as_root=run_as_root)
-
         if is_xenserver_format(image_meta):
             replace_xenserver_image_with_coalesced_vhd(tmp)
 
@@ -943,6 +799,21 @@ def fetch_to_volume_format(context: context.RequestContext,
                 reason=_("fmt=%(fmt)s backed by:%(backing_file)s")
                 % {'fmt': fmt, 'backing_file': backing_file, })
 
+        # NOTE(ZhengMa): This is used to do image decompression on image
+        # downloading with 'compressed' container_format. It is a
+        # transparent level between original image downloaded from
+        # Glance and Cinder image service. So the source file path is
+        # the same with destination file path.
+        if image_meta.get('container_format') == 'compressed':
+            LOG.debug("Found image with compressed container format")
+            if not accelerator.is_gzip_compressed(tmp):
+                raise exception.ImageUnacceptable(
+                    image_id=image_id,
+                    reason=_("Unsupported compressed image format found. "
+                             "Only gzip is supported currently"))
+            accel = accelerator.ImageAccel(tmp, tmp)
+            accel.decompress_img(run_as_root=run_as_root)
+
         # NOTE(jdg): I'm using qemu-img convert to write
         # to the volume regardless if it *needs* conversion or not
         # TODO(avishay): We can speed this up by checking if the image is raw
@@ -950,21 +821,13 @@ def fetch_to_volume_format(context: context.RequestContext,
         # check via 'qemu-img info' that what we copied was in fact a raw
         # image and not a different format with a backing file, which may be
         # malicious.
-        # FIXME: revisit the above 2 comments.  We already have an exception
-        # above for RAW format images when qemu-img is not available, and I'm
-        # pretty sure that the backing file exploit only happens when
-        # converting from some format that supports a backing file TO raw ...
-        # a bit-for-bit copy of a qcow2 with backing file will copy the backing
-        # file *reference* but not its *content*.
         disk_format = fixup_disk_format(image_meta['disk_format'])
         LOG.debug("%s was %s, converting to %s", image_id, fmt, volume_format)
 
         convert_image(tmp, dest, volume_format,
                       out_subformat=volume_subformat,
                       src_format=disk_format,
-                      run_as_root=run_as_root,
-                      image_id=image_id,
-                      data=data)
+                      run_as_root=run_as_root)
 
 
 def upload_volume(context: context.RequestContext,
@@ -974,11 +837,15 @@ def upload_volume(context: context.RequestContext,
                   volume_format: str = 'raw',
                   run_as_root: bool = True,
                   compress: bool = True,
-                  store_id: str = None,
+                  store_id: Optional[str] = None,
                   base_image_ref: Optional[str] = None) -> None:
     # NOTE: You probably want to use volume_utils.upload_volume(),
     # not this function.
     image_id = image_meta['id']
+
+    check_image_conversion_disable(
+        image_meta['disk_format'], volume_format, image_id, upload=True)
+
     if image_meta.get('container_format') != 'compressed':
         if (image_meta['disk_format'] == volume_format):
             LOG.debug("%s was %s, no need to convert to %s",
@@ -1018,9 +885,7 @@ def upload_volume(context: context.RequestContext,
         out_format = fixup_disk_format(image_meta['disk_format'])
         convert_image(volume_path, tmp, out_format,
                       run_as_root=run_as_root,
-                      compress=compress,
-                      image_id=image_id,
-                      data=data)
+                      compress=compress)
 
         data = qemu_img_info(tmp, run_as_root=run_as_root)
         if data.file_format != out_format:
@@ -1091,7 +956,7 @@ def extract_targz(archive_name: str, target: str) -> None:
     utils.execute('tar', '-xzf', archive_name, '-C', target)
 
 
-def fix_vhd_chain(vhd_chain: List[str]) -> None:
+def fix_vhd_chain(vhd_chain: list[str]) -> None:
     for child, parent in zip(vhd_chain[:-1], vhd_chain[1:]):
         set_vhd_parent(child, parent)
 
@@ -1160,7 +1025,7 @@ def temporary_dir() -> ContextManager[str]:
     return utils.tempdir(dir=CONF.image_conversion_dir)
 
 
-def coalesce_chain(vhd_chain: List[str]) -> str:
+def coalesce_chain(vhd_chain: list[str]) -> str:
     for child, parent in zip(vhd_chain[:-1], vhd_chain[1:]):
         with temporary_dir() as directory_for_journal:
             size = get_vhd_size(child)
@@ -1172,7 +1037,7 @@ def coalesce_chain(vhd_chain: List[str]) -> str:
     return vhd_chain[-1]
 
 
-def discover_vhd_chain(directory: str) -> List[str]:
+def discover_vhd_chain(directory: str) -> list[str]:
     counter = 0
     chain = []
 
@@ -1197,7 +1062,7 @@ def replace_xenserver_image_with_coalesced_vhd(image_file: str) -> None:
         os.rename(coalesced, image_file)
 
 
-def decode_cipher(cipher_spec: str, key_size: int) -> Dict[str, str]:
+def decode_cipher(cipher_spec: str, key_size: int) -> dict[str, str]:
     """Decode a dm-crypt style cipher specification string
 
        The assumed format being cipher-chainmode-ivmode, similar to that
@@ -1229,7 +1094,7 @@ class TemporaryImages(object):
     """
 
     def __init__(self, image_service: glance.GlanceImageService):
-        self.temporary_images: Dict[str, dict] = {}
+        self.temporary_images: dict[str, dict] = {}
         self.image_service = image_service
         image_service.temp_images = self
 
@@ -1268,39 +1133,3 @@ class TemporaryImages(object):
         if not self.temporary_images.get(user):
             return None
         return self.temporary_images[user].get(image_id)
-
-
-def _filter_out_metadata(metadata, filter_keys):
-    new_metadata = {}
-    for k, v in metadata.items():
-        if any(k.startswith(filter_key)
-               for filter_key in filter_keys):
-            continue
-        new_metadata[k] = v
-    return new_metadata
-
-
-def filter_out_reserved_namespaces_metadata(
-        metadata: Optional[Dict[str, str]]) -> Dict[str, str]:
-
-    reserved_name_spaces = GLANCE_RESERVED_NAMESPACES.copy()
-    if CONF.reserved_image_namespaces:
-        for image_namespace in CONF.reserved_image_namespaces:
-            if image_namespace not in reserved_name_spaces:
-                reserved_name_spaces.append(image_namespace)
-
-    if not metadata:
-        LOG.debug("No metadata to be filtered.")
-        return {}
-
-    new_metadata = _filter_out_metadata(metadata, reserved_name_spaces)
-    # NOTE(ganso): handle adjustment of metadata structure performed by
-    # the cinder.volume.api.API._merge_volume_image_meta() method
-    if 'properties' in new_metadata:
-        new_metadata['properties'] = _filter_out_metadata(
-            metadata['properties'], reserved_name_spaces)
-
-    LOG.debug("The metadata set [%s] was filtered using the reserved name "
-              "spaces [%s], and the result is [%s].", metadata,
-              reserved_name_spaces, new_metadata)
-    return new_metadata
