@@ -941,17 +941,16 @@ class QuobyteDriverTestCase(test.TestCase):
                                                self.TEST_QUOBYTE_VOLUME),
                                            self.VOLUME_UUID)
 
-        qemu_img_info_output = """{
-    "filename": "volume-%s",
-    "format": "qcow2",
-    "virtual-size": 1073741824,
-    "actual-size": 473000
-}""" % self.VOLUME_UUID
+        qemu_img_info_output = """image: volume-%s
+        file format: qcow2
+        virtual size: 1.0G (1073741824 bytes)
+        disk size: 473K
+        """ % self.VOLUME_UUID
 
-        img_info = imageutils.QemuImgInfo(qemu_img_info_output, format='json')
+        img_info = imageutils.QemuImgInfo(qemu_img_info_output)
 
-        image_utils.qemu_img_info = mock.Mock(return_value=img_info)
-        image_utils.resize_image = mock.Mock()
+        self.mock_object(image_utils, 'qemu_img_info', return_value=img_info)
+        self.mock_object(image_utils, 'resize_image')
 
         mock_remote_attached.return_value = is_attached
 
@@ -988,21 +987,20 @@ class QuobyteDriverTestCase(test.TestCase):
 
         size = dest_volume['size']
 
-        qemu_img_output = """{
-    "filename": "%s",
-    "format": "raw",
-    "virtual-size": 1073741824,
-    "actual-size": 173000,
-    "backing-filename": "%s"
-}""" % (snap_file, src_volume['name'])
-        img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
+        qemu_img_output = """image: %s
+        file format: raw
+        virtual size: 1.0G (1073741824 bytes)
+        disk size: 173K
+        backing file: %s
+        """ % (snap_file, src_volume['name'])
+        img_info = imageutils.QemuImgInfo(qemu_img_output)
 
         # mocking and testing starts here
-        image_utils.convert_image = mock.Mock()
+        mock_convert = self.mock_object(image_utils, 'convert_image')
         drv._read_info_file = mock.Mock(return_value=
                                         {'active': snap_file,
                                          snapshot['id']: snap_file})
-        image_utils.qemu_img_info = mock.Mock(return_value=img_info)
+        self.mock_object(image_utils, 'qemu_img_info', return_value=img_info)
         drv._set_rw_permissions = mock.Mock()
 
         drv._copy_volume_from_snapshot(snapshot, dest_volume, size)
@@ -1011,7 +1009,7 @@ class QuobyteDriverTestCase(test.TestCase):
         image_utils.qemu_img_info.assert_called_once_with(snap_path,
                                                           force_share=True,
                                                           run_as_root=False)
-        (image_utils.convert_image.
+        (mock_convert.
          assert_called_once_with(src_vol_path,
                                  dest_vol_path,
                                  'raw',
@@ -1045,21 +1043,20 @@ class QuobyteDriverTestCase(test.TestCase):
 
         size = dest_volume['size']
 
-        qemu_img_output = """{
-    "filename": "%s",
-    "format": "raw",
-    "virtual-size": 1073741824,
-    "actual-size": 173000,
-    "backing-filename": "%s"
-}""" % (snap_file, src_volume['name'])
-        img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
+        qemu_img_output = """image: %s
+        file format: raw
+        virtual size: 1.0G (1073741824 bytes)
+        disk size: 173K
+        backing file: %s
+        """ % (snap_file, src_volume['name'])
+        img_info = imageutils.QemuImgInfo(qemu_img_output)
 
         # mocking and testing starts here
-        image_utils.convert_image = mock.Mock()
+        mock_convert = self.mock_object(image_utils, 'convert_image')
         drv._read_info_file = mock.Mock(return_value=
                                         {'active': snap_file,
                                          snapshot['id']: snap_file})
-        image_utils.qemu_img_info = mock.Mock(return_value=img_info)
+        self.mock_object(image_utils, 'qemu_img_info', return_value=img_info)
         drv._set_rw_permissions = mock.Mock()
         shutil.copyfile = mock.Mock()
 
@@ -1069,7 +1066,7 @@ class QuobyteDriverTestCase(test.TestCase):
         image_utils.qemu_img_info.assert_called_once_with(snap_path,
                                                           force_share=True,
                                                           run_as_root=False)
-        self.assertFalse(image_utils.convert_image.called,
+        self.assertFalse(mock_convert.called,
                          ("_convert_image was called but should not have been")
                          )
         os_ac_mock.assert_called_once_with(
@@ -1107,21 +1104,20 @@ class QuobyteDriverTestCase(test.TestCase):
 
         size = dest_volume['size']
 
-        qemu_img_output = """{
-    "filename": "%s",
-    "format": "raw",
-    "virtual-size": 1073741824,
-    "actual-size": 173000,
-    "backing-filename": "%s"
-}""" % (snap_file, src_volume['name'])
-        img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
+        qemu_img_output = """image: %s
+        file format: raw
+        virtual size: 1.0G (1073741824 bytes)
+        disk size: 173K
+        backing file: %s
+        """ % (snap_file, src_volume['name'])
+        img_info = imageutils.QemuImgInfo(qemu_img_output)
 
         # mocking and testing starts here
-        image_utils.convert_image = mock.Mock()
+        mock_convert = self.mock_object(image_utils, 'convert_image')
         drv._read_info_file = mock.Mock(return_value=
                                         {'active': snap_file,
                                          snapshot['id']: snap_file})
-        image_utils.qemu_img_info = mock.Mock(return_value=img_info)
+        self.mock_object(image_utils, 'qemu_img_info', return_value=img_info)
         drv._set_rw_permissions = mock.Mock()
         drv._create_overlay_volume_from_snapshot = mock.Mock()
 
@@ -1133,7 +1129,7 @@ class QuobyteDriverTestCase(test.TestCase):
         image_utils.qemu_img_info.assert_called_once_with(snap_path,
                                                           force_share=True,
                                                           run_as_root=False)
-        (image_utils.convert_image.
+        (mock_convert.
          assert_called_once_with(
              src_vol_path,
              drv._local_volume_from_snap_cache_path(snapshot), 'qcow2',
@@ -1172,23 +1168,22 @@ class QuobyteDriverTestCase(test.TestCase):
 
         size = dest_volume['size']
 
-        qemu_img_output = """{
-    "filename": "%s",
-    "format": "raw",
-    "virtual-size": 1073741824,
-    "actual-size": 173000,
-    "backing-filename": "%s"
-}""" % (snap_file, src_volume['name'])
-        img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
+        qemu_img_output = """image: %s
+        file format: raw
+        virtual size: 1.0G (1073741824 bytes)
+        disk size: 173K
+        backing file: %s
+        """ % (snap_file, src_volume['name'])
+        img_info = imageutils.QemuImgInfo(qemu_img_output)
 
         # mocking and testing starts here
-        image_utils.convert_image = mock.Mock()
+        mock_convert = self.mock_object(image_utils, 'convert_image')
         drv._read_info_file = mock.Mock(return_value=
                                         {'active': snap_file,
                                          snapshot['id']: snap_file})
-        image_utils.qemu_img_info = mock.Mock(return_value=img_info)
+        self.mock_object(image_utils, 'qemu_img_info', return_value=img_info)
         drv._set_rw_permissions = mock.Mock()
-        shutil.copyfile = mock.Mock()
+        self.mock_object(shutil, 'copyfile')
 
         drv._copy_volume_from_snapshot(snapshot, dest_volume, size)
 
@@ -1196,7 +1191,7 @@ class QuobyteDriverTestCase(test.TestCase):
         image_utils.qemu_img_info.assert_called_once_with(snap_path,
                                                           force_share=True,
                                                           run_as_root=False)
-        (image_utils.convert_image.
+        (mock_convert.
          assert_called_once_with(
              src_vol_path,
              drv._local_volume_from_snap_cache_path(snapshot), 'raw',
@@ -1250,16 +1245,15 @@ class QuobyteDriverTestCase(test.TestCase):
                                drv._get_hash_str(self.TEST_QUOBYTE_VOLUME))
         vol_path = os.path.join(vol_dir, volume['name'])
 
-        qemu_img_output = """{
-    "filename": "%s",
-    "format": "raw",
-    "virtual-size": 1073741824,
-    "actual-size": 173000
-}""" % volume['name']
-        img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
+        qemu_img_output = """image: %s
+        file format: raw
+        virtual size: 1.0G (1073741824 bytes)
+        disk size: 173K
+        """ % volume['name']
+        img_info = imageutils.QemuImgInfo(qemu_img_output)
 
         drv.get_active_image_from_info = mock.Mock(return_value=volume['name'])
-        image_utils.qemu_img_info = mock.Mock(return_value=img_info)
+        self.mock_object(image_utils, 'qemu_img_info', return_value=img_info)
 
         conn_info = drv.initialize_connection(volume, None)
 
@@ -1300,13 +1294,12 @@ class QuobyteDriverTestCase(test.TestCase):
 
             mock_create_temporary_file.return_value = self.TEST_TMP_FILE
 
-            qemu_img_output = """{
-    "filename": "%s",
-    "format": "raw",
-    "virtual-size": 1073741824,
-    "actual-size": 173000
-}""" % volume['name']
-            img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
+            qemu_img_output = """image: %s
+            file format: raw
+            virtual size: 1.0G (1073741824 bytes)
+            disk size: 173K
+            """ % volume['name']
+            img_info = imageutils.QemuImgInfo(qemu_img_output)
             mock_qemu_img_info.return_value = img_info
 
             upload_path = volume_path
@@ -1353,13 +1346,12 @@ class QuobyteDriverTestCase(test.TestCase):
 
             mock_create_temporary_file.return_value = self.TEST_TMP_FILE
 
-            qemu_img_output = """{
-    "filename": "%s",
-    "format": "qcow2",
-    "virtual-size": 1073741824,
-    "actual-size": 173000
-}""" % volume['name']
-            img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
+            qemu_img_output = """image: %s
+            file format: qcow2
+            virtual size: 1.0G (1073741824 bytes)
+            disk size: 173K
+            """ % volume['name']
+            img_info = imageutils.QemuImgInfo(qemu_img_output)
             mock_qemu_img_info.return_value = img_info
 
             upload_path = self.TEST_TMP_FILE
@@ -1409,14 +1401,13 @@ class QuobyteDriverTestCase(test.TestCase):
 
             mock_create_temporary_file.return_value = self.TEST_TMP_FILE
 
-            qemu_img_output = """{
-    "filename": "volume-%s.%s",
-    "format": "qcow2",
-    "virtual-size": 1073741824,
-    "actual-size": 173000,
-    "backing-filename": "%s"
-}""" % (self.VOLUME_UUID, self.SNAP_UUID, volume_filename)
-            img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
+            qemu_img_output = """image: volume-%s.%s
+            file format: qcow2
+            virtual size: 1.0G (1073741824 bytes)
+            disk size: 173K
+            backing file: %s
+            """ % (self.VOLUME_UUID, self.SNAP_UUID, volume_filename)
+            img_info = imageutils.QemuImgInfo(qemu_img_output)
             mock_qemu_img_info.return_value = img_info
 
             upload_path = self.TEST_TMP_FILE
