@@ -85,9 +85,6 @@ class IBMStorageDriver(san.SanDriver,
     # ThirdPartySystems wiki page
     CI_WIKI_NAME = "IBM_STORAGE_CI"
 
-    # TODO(jsbryant) Remove driver in the 'U' release if CI is not fixed.
-    SUPPORTED = False
-
     def __init__(self, *args, **kwargs):
         """Initialize the driver."""
 
@@ -244,6 +241,10 @@ class IBMStorageDriver(san.SanDriver,
 
         return self.proxy.retype(ctxt, volume, new_type, diff, host)
 
+    def revert_to_snapshot(self, ctxt, volume, snapshot):
+        """Revert volume to snapshot."""
+        return self.proxy.revert_to_snapshot(ctxt, volume, snapshot)
+
     def create_group(self, context, group):
         """Creates a group."""
 
