@@ -16,7 +16,6 @@
 import functools
 import gettext
 import inspect
-import platform
 
 from oslo_log import log as logging
 from oslo_utils import timeutils
@@ -216,6 +215,11 @@ class IBMStorageProxy(object):
         """Convert the volume to be of the new type."""
         pass
 
+    @_trace_time
+    def revert_to_snapshot(self, context, volume, snapshot):
+        """Revert volume to snapshot."""
+        pass
+
     @_trace_time
     def _get_bunch_from_host(
             self, connector, host_id=0, host_name=None, chap=None):
@@ -235,12 +239,6 @@ class IBMStorageProxy(object):
                 'wwpns': wwpns,
                 'chap': chap}
 
-    @_trace_time
-    def _get_os_type(self):
-        """Gets a string representation of the current os"""
-        dist = platform.dist()
-        return "%s-%s-%s" % (dist[0], dist[1], platform.processor())
-
     def _log(self, level, message, **kwargs):
         """Wrapper around the logger"""
         to_log = _(self.prefix + message)  # NOQA
