@@ -25,7 +25,12 @@ from eventlet import greenthread
 from eventlet import semaphore
 from lxml import etree
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from oslo_utils import netutils
+import requests
+from requests.adapters import HTTPAdapter
+from requests import auth
+from requests.packages.urllib3.util.retry import Retry
 import six
 from six.moves import urllib
 
@@ -37,6 +42,7 @@ from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 
+# ZAPI API error codes.
 EAPIERROR = '13001'
 EAPIPRIVILEGE = '13003'
 EAPINOTFOUND = '13005'
@@ -342,11 +348,6 @@ class NaElement(object):
         """Add the attribute to the element."""
         self._element.set(name, value)
 
-    def add_attrs(self, **attrs):
-        """Add multiple attributes to the element."""
-        for attr in attrs:
-            self._element.set(attr, attrs.get(attr))
-
     def add_child_elem(self, na_element):
         """Add the child element to the element."""
         if isinstance(na_element, NaElement):
@@ -554,6 +555,12 @@ class NaApiError(Exception):
         return 'NetApp API failed. Reason - %s:%s' % (self.code, self.message)
 
 
+class NaRetryableError(NaApiError):
+    def __str__(self, *args, **kwargs):
+        return 'NetApp API failed. Try again. Reason - %s:%s' % (
+            self.code, self.message)
+
+
 class SSHUtil(object):
     """Encapsulates connection logic and command execution for SSH client."""
 
@@ -601,7 +608,7 @@ class SSHUtil(object):
         stdin, stdout, stderr = client.exec_command(command)
         self._wait_on_stdout(stdout, timeout)
         response = stdout.channel.recv(999)
-        if response.strip() != expected_prompt_text:
+        if expected_prompt_text not in response.strip().decode():
             msg = _("Unexpected output. Expected [%(expected)s] but "
                     "received [%(output)s]") % {
                 'expected': expected_prompt_text,
@@ -633,3 +640,234 @@ class SSHUtil(object):
             if wait_time > timeout:
                 LOG.debug("Timeout exceeded while waiting for exit status.")
                 break
+
+
+# REST API error codes.
+REST_UNAUTHORIZED = '6'
+REST_API_NOT_FOUND = '3'
+REST_UPDATE_SNAPMIRROR_FAILED = '13303844'
+REST_ERELATION_EXISTS = '6619637'
+REST_SNAPMIRROR_IN_PROGRESS = '13303810'
+REST_UPDATE_SNAPMIRROR_FAILED = '13303844'
+REST_NO_SUCH_LUN_MAP = '5374922'
+REST_NO_SUCH_FILE = '6684674'
+
+
+class RestNaServer(object):
+
+    TRANSPORT_TYPE_HTTP = 'http'
+    TRANSPORT_TYPE_HTTPS = 'https'
+    HTTP_PORT = '80'
+    HTTPS_PORT = '443'
+
+    TRANSPORT_PORT = {
+        TRANSPORT_TYPE_HTTP: HTTP_PORT,
+        TRANSPORT_TYPE_HTTPS: HTTPS_PORT
+    }
+
+    def __init__(self, host, transport_type=TRANSPORT_TYPE_HTTP,
+                 ssl_cert_path=None, username=None, password=None, port=None,
+                 api_trace_pattern=None):
+        self._host = host
+        self.set_transport_type(transport_type)
+        self.set_port(port=port)
+        self._username = username
+        self._password = password
+
+        if api_trace_pattern is not None:
+            na_utils.setup_api_trace_pattern(api_trace_pattern)
+
+        if ssl_cert_path is not None:
+            self._ssl_verify = ssl_cert_path
+        else:
+            # Note(felipe_rodrigues): it will verify with the Mozila CA roots,
+            # given by certifi package.
+            self._ssl_verify = True
+
+        self._api_version = None
+        self._api_major_version = None
+        self._api_minor_version = None
+        self._ontap_version = None
+        self._timeout = None
+
+        LOG.debug('Using REST with NetApp controller: %s', self._host)
+
+    def set_transport_type(self, transport_type):
+        """Set the transport type protocol for API.
+
+        Supports http and https transport types.
+        """
+        if transport_type is None or transport_type.lower() not in (
+                RestNaServer.TRANSPORT_TYPE_HTTP,
+                RestNaServer.TRANSPORT_TYPE_HTTPS):
+            raise ValueError('Unsupported transport type')
+        self._protocol = transport_type.lower()
+
+    def get_transport_type(self):
+        """Get the transport type protocol."""
+        return self._protocol
+
+    def set_api_version(self, major, minor):
+        """Set the API version."""
+        try:
+            self._api_major_version = int(major)
+            self._api_minor_version = int(minor)
+            self._api_version = str(major) + "." + str(minor)
+        except ValueError:
+            raise ValueError('Major and minor versions must be integers')
+
+    def get_api_version(self):
+        """Gets the API version tuple."""
+        if not self._api_version:
+            return None
+        return (self._api_major_version, self._api_minor_version)
+
+    def set_ontap_version(self, ontap_version):
+        """Set the ONTAP version."""
+        self._ontap_version = ontap_version
+
+    def get_ontap_version(self):
+        """Gets the ONTAP version."""
+        return self._ontap_version
+
+    def set_port(self, port=None):
+        """Set the ONTAP port, if not informed, set with default one."""
+        if port is None and self._protocol in RestNaServer.TRANSPORT_PORT:
+            self._port = RestNaServer.TRANSPORT_PORT[self._protocol]
+        else:
+            try:
+                int(port)
+            except ValueError:
+                raise ValueError('Port must be integer')
+            self._port = str(port)
+
+    def get_port(self):
+        """Get the server communication port."""
+        return self._port
+
+    def set_timeout(self, seconds):
+        """Sets the timeout in seconds."""
+        try:
+            self._timeout = int(seconds)
+        except ValueError:
+            raise ValueError('timeout in seconds must be integer')
+
+    def get_timeout(self):
+        """Gets the timeout in seconds if set."""
+        return self._timeout
+
+    def set_vserver(self, vserver):
+        """Set the vserver to use if tunneling gets enabled."""
+        self._vserver = vserver
+
+    def get_vserver(self):
+        """Get the vserver to use in tunneling."""
+        return self._vserver
+
+    def __str__(self):
+        """Gets a representation of the client."""
+        return "server: %s" % (self._host)
+
+    def _get_request_method(self, method, session):
+        """Returns the request method to be used in the REST call."""
+
+        request_methods = {
+            'post': session.post,
+            'get': session.get,
+            'put': session.put,
+            'delete': session.delete,
+            'patch': session.patch,
+        }
+        return request_methods[method]
+
+    def _add_query_params_to_url(self, url, query):
+        """Populates the URL with specified filters."""
+        filters = '&'.join([f"{k}={v}" for k, v in query.items()])
+        url += "?" + filters
+        return url
+
+    def _get_base_url(self):
+        """Get the base URL for REST requests."""
+        host = self._host
+        if ':' in host:
+            host = '[%s]' % host
+        return '%s://%s:%s/api/' % (self._protocol, host, self._port)
+
+    def _build_session(self, headers):
+        """Builds a session in the client."""
+        self._session = requests.Session()
+
+        # NOTE(felipe_rodrigues): request resilient of temporary network
+        # failures (like name resolution failure), retrying until 5 times.
+        max_retries = Retry(total=5, connect=5, read=2, backoff_factor=1)
+        adapter = HTTPAdapter(max_retries=max_retries)
+        self._session.mount('%s://' % self._protocol, adapter)
+
+        self._session.auth = self._create_basic_auth_handler()
+        self._session.verify = self._ssl_verify
+        self._session.headers = headers
+
+    def _build_headers(self, enable_tunneling):
+        """Build and return headers for a REST request."""
+        headers = {
+            "Accept": "application/json",
+            "Content-Type": "application/json"
+        }
+        if enable_tunneling:
+            headers["X-Dot-SVM-Name"] = self.get_vserver()
+
+        return headers
+
+    def _create_basic_auth_handler(self):
+        """Creates and returns a basic HTTP auth handler."""
+        return auth.HTTPBasicAuth(self._username, self._password)
+
+    @volume_utils.trace_api(
+        filter_function=na_utils.trace_filter_func_rest_api)
+    def send_http_request(self, method, url, body, headers):
+        """Invoke the API on the server.
+
+        The passed parameters and returned parameters will be logged if trace
+        feature is on. They are important for debugging purpose.
+        """
+        data = jsonutils.dumps(body) if body else {}
+
+        self._build_session(headers)
+        request_method = self._get_request_method(method, self._session)
+
+        try:
+            if self._timeout is not None:
+                response = request_method(
+                    url, data=data, timeout=self._timeout)
+            else:
+                response = request_method(url, data=data)
+        except requests.HTTPError as e:
+            raise NaApiError(e.errno, e.strerror)
+        except Exception as e:
+            raise NaApiError(message=e)
+
+        code = response.status_code
+        body = jsonutils.loads(response.content) if response.content else {}
+        return code, body
+
+    def invoke_successfully(self, action_url, method, body=None, query=None,
+                            enable_tunneling=False):
+        """Invokes REST API and checks execution status as success."""
+        headers = self._build_headers(enable_tunneling)
+        if query:
+            action_url = self._add_query_params_to_url(action_url, query)
+        url = self._get_base_url() + action_url
+        code, response = self.send_http_request(method, url, body, headers)
+
+        if not response.get('error'):
+            return code, response
+
+        result_error = response.get('error')
+        code = result_error.get('code', 'ESTATUSFAILED')
+        # TODO: add the correct code number for REST not licensed clone error.
+        if code == ESIS_CLONE_NOT_LICENSED:
+            msg = 'Clone operation failed: FlexClone not licensed.'
+        else:
+            msg = (result_error.get('message')
+                   or 'Execution status is failed due to unknown reason')
+        raise NaApiError(code, msg)
